/* MUI multi column list view example                                         */

/*----------------------------------------------------------------------------*/
/* System Includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>

#if !defined(__VBCC__) && defined(__MORPHOS__)
	#define __NOLIBBASE__
#endif
	#include <proto/intuition.h>
	#include <proto/muimaster.h>

	#include <libraries/mui.h>
	#include <libraries/iffparse.h> /* for MAKE_ID */

	#include <clib/alib_protos.h>

/*----------------------------------------------------------------------------*/
/* Constants & Macros                                                         */
/*----------------------------------------------------------------------------*/

	#define ID_USE 1

/*----------------------------------------------------------------------------*/
/* Global Variables                                                           */
/*----------------------------------------------------------------------------*/

#ifndef __VBCC__
	struct Library *MUIMasterBase;
#ifdef __amigaos4__
	struct MUIMasterIFace *IMUIMaster;
#endif
#ifdef __MORPHOS__
	struct Library *IntuitionBase;
#endif
#endif

	struct item {
		const char *name;
		const char *amount;
		} list[] = {
	{"Beer","1,000"},
	{"Juice","5,000"},
	{"Water","20,000"},
	{"Wine","500"},
	{NULL}};

/*----------------------------------------------------------------------------*/
/* Initialize a callback hook                                                 */
/*----------------------------------------------------------------------------*/

	void init_hook (struct Hook *hook,HOOKFUNC entry,APTR data)

	{
#ifdef __amigaos4__
	hook->h_Entry    = (HOOKFUNC)(entry);
#else
	hook->h_Entry    = (HOOKFUNC)HookEntry;
	hook->h_SubEntry = (HOOKFUNC)entry;
#endif
	hook->h_Data     = (APTR)data;
	}

/*----------------------------------------------------------------------------*/
/* Display function for list view                                             */
/*----------------------------------------------------------------------------*/

	LONG dispfunct (struct Hook *hook, const char **array, struct item *item)

	{
	if (item)
		{
		array[0] = item->name;
		array[1] = item->amount;
		}
	else
		{
		array[0] = "Name";
		array[1] = "Amount";
		}
	return (0);
	}

/*----------------------------------------------------------------------------*/
/* Main Routine                                                               */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	Object *app;
	Object *win;
	Object *lv;
	Object *btuse;
	Object *btcancel;
	struct Hook disphook;

#ifndef __VBCC__
	if (!(MUIMasterBase = OpenLibrary ("muimaster.library",20)))
		return (RETURN_FAIL);
#ifdef __amigaos4__
	if (!(IMUIMaster = (struct MUIMasterIFace *) GetInterface (MUIMasterBase,"main",1,NULL)))
		{
		CloseLibrary (MUIMasterBase);
		return (RETURN_FAIL);
		}
#endif
#ifdef __MORPHOS__
	if (!(IntuitionBase = OpenLibrary ("intuition.library",36)))
		{
		CloseLibrary (MUIMasterBase);
		return (RETURN_FAIL);
		}
#endif
#endif

	init_hook (&disphook, (HOOKFUNC)dispfunct, NULL);

	app = ApplicationObject,
		MUIA_Application_Title      , "List Example",
		MUIA_Application_Base       , "LISTEXAMPLE",

		SubWindow, win = WindowObject,
			MUIA_Window_Title, "List Example",
			MUIA_Window_ID   , MAKE_ID('L','S','T','X'),

			WindowContents, VGroup,

				Child, ListviewObject,
					MUIA_Listview_MultiSelect, MUIV_Listview_MultiSelect_Default,
					MUIA_Listview_List, lv = ListObject,
						InputListFrame,
						MUIA_List_Format, "MIW=60 BAR,MIW=30 P=\x1br",
						MUIA_List_DisplayHook, &disphook,
						MUIA_List_Title, TRUE,
						End,
					End,

				Child, VSpace(2),

				Child, HGroup,
					MUIA_Group_SameSize, TRUE,
					Child, btuse    = SimpleButton("Use"),
					Child, btcancel = SimpleButton("Cancel"),
					End,

				End,
			End,
		End;

	if (app)
		{
		ULONG muisig = 0;
		BOOL cont = TRUE;
		long num;

		set (btuse,MUIA_ControlChar,'\r');
		set (btcancel,MUIA_ControlChar,'<');

		DoMethod (btcancel,MUIM_Notify,MUIA_Pressed,FALSE,app,2,MUIM_Application_ReturnID,MUIV_Application_ReturnID_Quit);
		DoMethod (btuse,MUIM_Notify,MUIA_Pressed,FALSE,app,2,MUIM_Application_ReturnID,ID_USE);
		DoMethod (lv,MUIM_Notify,MUIA_Listview_DoubleClick,TRUE,app,2,MUIM_Application_ReturnID,ID_USE);
		DoMethod (win,MUIM_Notify,MUIA_Window_CloseRequest,TRUE,app,2,MUIM_Application_ReturnID,MUIV_Application_ReturnID_Quit);

		{
		struct item *item;
		for (item = list; item->name; item++)
			DoMethod (lv,MUIM_List_InsertSingle,item,MUIV_List_Insert_Bottom);
		}

		set (win,MUIA_Window_Open,TRUE);

		do	{
			if (muisig)
				{
				if (Wait (muisig | SIGBREAKF_CTRL_C) & SIGBREAKF_CTRL_C)
					cont = FALSE;
				}

			switch (DoMethod (app,MUIM_Application_Input,&muisig))
				{
			case MUIV_Application_ReturnID_Quit:
				cont = FALSE;
				break;

			case ID_USE:
				num = MUIV_List_Active_Off;
				get (lv,MUIA_List_Active,&num);
				if (num == MUIV_List_Active_Off)
					DisplayBeep (NULL);
				else
					{
					Printf ("%s\n",list[num].name);
					cont = FALSE;
					}
				break;
				}
			}
		while (cont);

		MUI_DisposeObject(app);
		}

#ifndef __VBCC__
#ifdef __MORPHOS__
	CloseLibrary (IntuitionBase);
#endif
#ifdef __amigaos4__
	DropInterface ((struct Interface *)IMUIMaster);
#endif
	CloseLibrary (MUIMasterBase);
#endif

	return (RETURN_OK);
	}

/*----------------------------------------------------------------------------*/
/* End of source text                                                         */
/*----------------------------------------------------------------------------*/
